/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:19:06.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.app.core.SysProperty;
import cn.com.qmth.examcloud.app.core.exception.ApiException;
import cn.com.qmth.examcloud.app.core.utils.HttpClientBuilder;
import cn.com.qmth.examcloud.app.core.utils.HttpUtils;
import cn.com.qmth.examcloud.app.core.utils.ThreadUtils;
import cn.com.qmth.examcloud.app.model.*;
import cn.com.qmth.examcloud.app.service.CoreAuthService;
import cn.com.qmth.examcloud.app.service.RedisService;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.support.CacheConstants;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.OrgPropertyCacheBean;
import com.alibaba.fastjson.JSONObject;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.Map;

import static cn.com.qmth.examcloud.app.model.Constants.*;

/**
 * 认证中心业务服务接口
 *
 * @author: QMTH
 * @since: 2018/7/16
 */
@Service
public class CoreAuthServiceImpl implements CoreAuthService {

    private static final Logger log = LoggerFactory.getLogger(CoreAuthServiceImpl.class);

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private RedisService redisService;

    @Autowired
    private StudentCloudService studentCloudService;

    @Override
    public boolean isDoingExam(Long rootOrgId, String accountType, String account) {
        if (rootOrgId == null) {
            log.warn("[Check Doing Exam] rootOrgId is null");
            return false;
        }

        log.info(String.format("[Check Doing Exam] rootOrgId=%s, account=%s, accountType=%s", rootOrgId, account, accountType));

        Map<String, String> params = new HashMap<>();
        params.put("rootOrgId", rootOrgId.toString());
        if (LoginType.STUDENT_CODE.name().equals(accountType)) {
            params.put("studentCode", account);
        } else if (LoginType.STUDENT_IDENTITY_NUMBER.name().equals(accountType)) {
            params.put("identityNumber", account);
        } else {
            //LoginType.STUDENT_PHONE
            params.put("phone", account);
        }

        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/getStudentOnLineExamInfo", sysProperty.getApiDomain());
        try {
            //执行请求
            String json = new JsonMapper().toJson(params);
            RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
            Request request = new Request.Builder()
                    .post(formBody)
                    .url(requestUrl)
                    .build();
            Response response = HttpClientBuilder.getClient().newCall(request).execute();

            String bodyStr = response.body().string();
            if (response.isSuccessful()) {
                ExamInfo info = new JsonMapper().parseJson(bodyStr, ExamInfo.class);
                if (info != null) {
                    return info.getExistExamingRecord();
                }
            } else {
                log.warn(bodyStr);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }

        log.warn("[Check Doing Exam] fail");
        return false;
    }

    @Override
    public boolean isOpenApp(Long rootOrgId) {
        if (rootOrgId == null) {
            log.warn("[Check Open APP] rootOrgId is null");
            return false;
        }

        try {
            OrgPropertyCacheBean property = CacheHelper.getOrgProperty(rootOrgId, "APP_ENABLED");

            if ("true".equals(property.getValue())) {
                return true;
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return false;
    }

    @Override
    public Long getRootOrgIdBySecurityPhone(String securityPhone) {
        GetStudentReq req = new GetStudentReq();
        req.setSecurityPhone(securityPhone);

        try {
            GetStudentResp resp = studentCloudService.getStudent(req);
            return resp.getStudentInfo().getRootOrgId();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }

        return null;
    }

    @Override
    public Result<UserInfo> login(LoginInfo loginInfo) {
        Assert.notNull(loginInfo, "LoginInfo must be not null.");
        if (StringUtils.isBlank(loginInfo.getAccountType())) {
            loginInfo.setAccountType(LoginType.STUDENT_PHONE.name());
        }

        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/login", sysProperty.getApiDomain());
        Map<String, String> params = new HashMap<>();
        params.put("accountValue", loginInfo.getAccount());
        params.put("password", loginInfo.getPassword());
        params.put("accountType", loginInfo.getAccountType());
        params.put("rootOrgId", loginInfo.getRootOrgId() != null ? loginInfo.getRootOrgId().toString() : "");
        params.put("domain", loginInfo.getDomain());
        params.put("smsCode", loginInfo.getSmsCode());
        if (loginInfo.getNoSession() != null && loginInfo.getNoSession()) {
            params.put("noSession", loginInfo.getNoSession().toString());
        }
        String json = new JsonMapper().toJson(params);

        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        Request request = new Request.Builder()
                .url(requestUrl)
                .post(formBody)
                .addHeader("isApp", "true")
                .build();

        //执行请求
        try (Response response = HttpClientBuilder.getClient().newCall(request).execute();) {
            String bodyStr = response.body().string();
            if (response.isSuccessful()) {
                //获取用户信息
                UserInfo userInfo = new JsonMapper().parseJson(bodyStr, UserInfo.class);
                return new Result().success(userInfo);
            } else {
                log.warn("Http response is " + bodyStr);
                ResBody body = new JsonMapper().parseJson(bodyStr, ResBody.class);
                if (body != null && body.getCode() != null) {
                    return new Result().error(body.getDesc());
                }
                return new Result().error(bodyStr);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new Result().error("登录异常！");
        }
    }

    @Override
    public Result logout(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/logout", sysProperty.getApiDomain());
        RequestBody formBody = new FormBody.Builder()
                .add(PARAM_KEY, key)
                .add(PARAM_TOKEN, token)
                .build();

        //清除缓存用户登录信息
        this.removeCacheLoginInfo(key);

        //退出登录
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result updateStudentPassword(String key, String token, String password, String newPassword) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/student/password", sysProperty.getApiDomain());
        RequestBody formBody = new FormBody.Builder()
                .add("password", password)
                .add("newPassword", newPassword)
                .build();
        return HttpUtils.doPut(requestUrl, formBody, key, token);
    }

    @Override
    public Result resetStudentPassword(String key, String token, String newPassword) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/student/password/direct", sysProperty.getApiDomain());
        RequestBody formBody = new FormBody.Builder()
                .add("newPassword", newPassword)
                .build();
        return HttpUtils.doPut(requestUrl, formBody, key, token);
    }

    @Override
    public Result sendSmsCode(String key, String token, String phone) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/sendVerificationCode", sysProperty.getApiDomain());
        RequestBody formBody = new FormBody.Builder()
                .add("phone", phone)
                .build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result code4Student(String phone, Boolean bound) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/sendVerificationCode4Student", sysProperty.getApiDomain());
        RequestBody formBody = new FormBody.Builder()
                .add("phone", phone)
                .add("bound", bound.toString())
                .build();
        Request request = new Request.Builder()
                .url(requestUrl)
                .post(formBody)
                .build();

        //执行请求
        String traceId = HttpUtils.getRandomTraceId();
        log.info("[TRACE_ID] " + traceId + " [POST] " + requestUrl);
        return HttpUtils.call(request, traceId);
    }

    @Override
    public Result userBindingPhone(String key, String token, String phone, String code) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/auth/bindSecurityPhone", sysProperty.getApiDomain());
        RequestBody formBody = new FormBody.Builder()
                .add("phone", phone)
                .add("verificationCode", code)
                .build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    /**
     * 缓存用户登录信息
     */
    @Override
    public void cacheLoginInfo(LoginInfo loginInfo, String key) {
        if (StringUtils.isEmpty(key)) {
            throw new ApiException("key must be not empty.");
        }
        String jsonStr = new JsonMapper().toJson(loginInfo);
        redisService.set(CacheConstants.CACHE_APP_LOGIN_SESSION + key, jsonStr, APP_SESSION_EXPIRE_TIME);
    }

    /**
     * 清除缓存用户登录信息
     */
    private void removeCacheLoginInfo(String key) {
        if (StringUtils.isEmpty(key)) {
            throw new ApiException("key must be not empty.");
        }
        redisService.delete(CacheConstants.CACHE_APP_LOGIN_SESSION + key);
    }

    /**
     * 获取缓存中的用户登录信息
     */
    @Override
    public LoginInfo getLoginInfo(String key) {
        if (StringUtils.isEmpty(key)) {
            throw new ApiException("key must be not empty.");
        }
        return redisService.fromJson(CacheConstants.CACHE_APP_LOGIN_SESSION + key, LoginInfo.class);
    }

    /**
     * 初始化内部接口请求鉴权
     */
    @Override
    public void initRequestTrace() {
        String key = CacheConstants.CACHE_APP_REQ_TRACE + ThreadUtils.getTraceID();
        Long millis = System.currentTimeMillis();
        redisService.set(key, millis.toString(), 15);
    }

    @Override
    public Result register(String user_id, String client_type, String ip_address) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_core/verifyCode/register", sysProperty.getApiDomain());
        Map<String, Object> params = new HashMap<>();
        params.put("user_id", user_id);
        params.put("client_type", client_type);
        String json = new JsonMapper().toJson(params);
        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        //发送短信
        Result<String> result = HttpUtils.doPost(requestUrl, formBody);
        return result;
    }

    @Override
    public Result geetestLogin(String seccode, String validate, String challenge,
                               String user_id, String client_type, String ip_address,
                               String accountValue, String password, String accountType,
                               Long rootOrgId) throws Exception {
        if (StringUtils.isBlank(accountType)) {
            accountType = LoginType.STUDENT_PHONE.name();
        }

        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_core/verifyCode/gt/login", sysProperty.getApiDomain());
        Map<String, String> params = new HashMap<>();


        params.put("seccode", seccode);
        params.put("validate", validate);
        params.put("challenge", challenge);
        params.put("user_id", user_id);
        params.put("client_type", client_type);
        params.put("ip_address", ip_address);
        params.put("accountValue", accountValue);
        params.put("password", password);
        params.put("accountType", accountType);
        params.put("rootOrgId", rootOrgId != null ? rootOrgId.toString() : "");
        String json = new JsonMapper().toJson(params);

        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        Request request = new Request.Builder()
                .url(requestUrl)
                .post(formBody)
                .addHeader("isApp", "true")
                .build();

        //执行请求
        try (Response response = HttpClientBuilder.getClient().newCall(request).execute();) {
            String bodyStr = response.body().string();
            if (response.isSuccessful()) {
                //获取用户信息
                UserInfo userInfo = of(bodyStr);
                if (userInfo == null) {
                    log.warn("Login response is " + bodyStr);
                    return new Result<>().error("登录失败！");
                }

                return new Result().success(userInfo);
            } else {
                log.warn("Login response is " + bodyStr);
                ResBody body = new JsonMapper().parseJson(bodyStr, ResBody.class);
                if (body != null && body.getCode() != null) {
                    return new Result().error(body.getDesc());
                }
                return new Result().error(bodyStr);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new Result().error("登录失败！");
        }
    }

    private UserInfo of(String bodyStr) {
        JSONObject job = JSONObject.parseObject(bodyStr);
        JSONObject content = job.getJSONObject("content");
        if (content == null) {
            return null;
        }

        UserInfo userInfo = new UserInfo();
        userInfo.setKey(content.getString("key"));
        userInfo.setUserId(content.getLong("userId"));
        userInfo.setUserType(content.getString("userType"));
        userInfo.setDisplayName(content.getString("displayName"));
        userInfo.setRootOrgId(content.getLong("rootOrgId"));
        userInfo.setRootOrgName(content.getString("rootOrgName"));
        userInfo.setToken(content.getString("token"));
        return userInfo;
    }

}